local ADDON_NAME, PRIVATE_TABLE = ...

local _G = getfenv(0)
local UIParent = _G.UIParent
local WorldFrame = _G.WorldFrame

local GetTime = _G.GetTime
local UnitGUID = _G.UnitGUID
local IsMounted = _G.IsMounted
local CanLootUnit = _G.CanLootUnit
local CreateFrame = _G.CreateFrame
local MouselookStop = _G.MouselookStop
local IsMouselooking = _G.IsMouselooking
local InCombatLockdown = _G.InCombatLockdown
local IsEquippedItemType = _G.IsEquippedItemType
local CanScanResearchSite = _G.CanScanResearchSite
local ClearOverrideBindings = _G.ClearOverrideBindings
local SetOverrideBindingClick = _G.SetOverrideBindingClick

local LibStub = _G.LibStub
local AceAddon = LibStub("AceAddon-3.0")

PRIVATE_TABLE.addon = PRIVATE_TABLE.addon or {}

local BUTTON_NAME = "LootARangMatic"
local ACTION_DOUBLE_WAIT = 0.4
local MIN_ACTION_DOUBLECLICK = 0.05
local LOOT_A_RANG_1_ID = 60854
local LOOT_A_RANG_2_ID = 109167
local FETCH_SPELL_ID = 125050

--settings to be
local onlyLootTargetOrMouseover = false
local onlyLootTargetOrMouseoverInDigsite = true
local onlyLootTargetOrMouseoverFishing = true

local Addon = AceAddon:NewAddon(PRIVATE_TABLE.addon, ADDON_NAME, "AceEvent-3.0", "AceHook-3.0")

--This is based on the idea that Fishing Poles are the 17th listed Weapon type on the AH
local FISHING_POLE_NAME = select(17, GetAuctionItemSubClasses(1))

local function HoldingFishingPole()
	return IsEquippedItemType(FISHING_POLE_NAME)
end

local function InCombat()
	return PRIVATE_TABLE.inCombat or InCombatLockdown()
end

local function CheckForLoot()
	return onlyLootTargetOrMouseover
		or (onlyLootTargetOrMouseoverInDigsite and CanScanResearchSite())
		or (onlyLootTargetOrMouseoverFishing and HoldingFishingPole())
end

local function TargetOrMouseoverHasLoot()
	local targetGUID, mouseoverGUID = UnitGUID("target"), UnitGUID("mouseover")
	return (targetGUID and CanLootUnit(targetGUID)) or (mouseoverGUID and CanLootUnit(mouseoverGUID))
end

local function SafeToLoot()
	if PRIVATE_TABLE.moveing or IsMounted() or InCombat() then
		return false
	end
	if CheckForLoot() and not TargetOrMouseoverHasLoot() then
		return false
	end
	return true
end

function Addon:WantsToLoot()
	return SafeToLoot() and (GetItemCount(LOOT_A_RANG_1_ID, false, false) >= 1 or GetItemCount(LOOT_A_RANG_2_ID, false, false) >= 1 or IsSpellKnown(FETCH_SPELL_ID))
end

function Addon:RequestSkip()
	PRIVATE_TABLE.skipLoot = true
end

function Addon:OnInitialize()
	if not PRIVATE_TABLE.secureActionButton then
		local button = CreateFrame("Button", BUTTON_NAME, UIParent, "SecureActionButtonTemplate")
		button:Hide()
		button:EnableMouse(true)
		button:RegisterForClicks("RightButtonUp")
		button:SetAttribute("action", nil)
		button:SetScript("PostClick", function(self, mouse_button, is_down)
			if PRIVATE_TABLE.bindingOn and not InCombat() then
				ClearOverrideBindings(self)
				PRIVATE_TABLE.bindingOn = false
			else
				PRIVATE_TABLE.bindingStuck = true
			end
		end)
		
		PRIVATE_TABLE.secureActionButton = button
	end
end

function Addon:OnEnable()
	self:RegisterEvent("PLAYER_REGEN_DISABLED", "OnRegenDisabled")
	self:RegisterEvent("PLAYER_REGEN_ENABLED", "OnRegenEnabled")
	self:RegisterEvent("PLAYER_STARTED_MOVING", "OnPlayerMoveStart")
	self:RegisterEvent("PLAYER_STOPPED_MOVING", "OnPlayerMoveStop")
	self:SecureHookScript(WorldFrame, "OnMouseDown", "OnMouseDown")
end

function Addon:OnDisable()
	self:UnregisterEvent("PLAYER_REGEN_DISABLED")
	self:UnregisterEvent("PLAYER_REGEN_ENABLED")
	self:UnregisterEvent("PLAYER_STARTED_MOVING")
	self:UnregisterEvent("PLAYER_STOPPED_MOVING")
	self:Unhook(WorldFrame, "OnMouseDown")
end

function Addon:OnRegenDisabled()
	PRIVATE_TABLE.inCombat = true
end

function Addon:OnRegenEnabled()
	PRIVATE_TABLE.inCombat = false
	if PRIVATE_TABLE.bindingStuck then
		ClearOverrideBindings(PRIVATE_TABLE.secureActionButton)
		PRIVATE_TABLE.bindingStuck = false
		PRIVATE_TABLE.bindingStuck = false
	end
end

function Addon:OnPlayerMoveStart()
	PRIVATE_TABLE.moveing = true
end

function Addon:OnPlayerMoveStop()
	PRIVATE_TABLE.moveing = false
end

function Addon:OnMouseDown(frame, button, down)
	if button == "RightButton" then
		if PRIVATE_TABLE.lastClicked then
			local doubleTime = GetTime() - PRIVATE_TABLE.lastClicked
			if doubleTime < ACTION_DOUBLE_WAIT and doubleTime > MIN_ACTION_DOUBLECLICK then
				PRIVATE_TABLE.lastClicked = nil
				if SafeToLoot() then
					if PRIVATE_TABLE.skipLoot then
						PRIVATE_TABLE.skipLoot = false
					else
						local name, bind = "", true
						if GetItemCount(LOOT_A_RANG_1_ID, false, false) >= 1 then
							PRIVATE_TABLE.secureActionButton:SetAttribute("type", "item")
							PRIVATE_TABLE.secureActionButton:SetAttribute("item", select(1, GetItemInfo(LOOT_A_RANG_1_ID)))
						elseif GetItemCount(LOOT_A_RANG_2_ID, false, false) >= 1 then
							PRIVATE_TABLE.secureActionButton:SetAttribute("type", "item")
							PRIVATE_TABLE.secureActionButton:SetAttribute("item", select(1, GetItemInfo(LOOT_A_RANG_2_ID)))
						elseif IsSpellKnown(FETCH_SPELL_ID) then
							PRIVATE_TABLE.secureActionButton:SetAttribute("type", "spell")
							PRIVATE_TABLE.secureActionButton:SetAttribute("spell", select(1, GetSpellInfo(FETCH_SPELL_ID)))
						else
							bind = false;
						end
						if bind then
							if IsMouselooking() then
								MouselookStop()
							end
							SetOverrideBindingClick(PRIVATE_TABLE.secureActionButton, true, "BUTTON2", BUTTON_NAME)
							PRIVATE_TABLE.bindingOn = true
							PRIVATE_TABLE.setBindingOn = true
						end
					end
				end
			elseif doubleTime >= ACTION_DOUBLE_WAIT then
				PRIVATE_TABLE.lastClicked = GetTime()
			end
		else
			PRIVATE_TABLE.lastClicked = GetTime()
		end
	 end
end